/* filter.c -- LZO packer

   This file is part of the LZO real-time data compression package.

   Copyright (C) 1997 Markus Franz Xaver Johannes Oberhumer
   Copyright (C) 1996 Markus Franz Xaver Johannes Oberhumer

   The LZO library and packer is free software; you can redistribute it
   and/or modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.
   If not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Markus F.X.J. Oberhumer
   markus.oberhumer@jk.uni-linz.ac.at
 */


#include "lzopack.h"


/*************************************************************************
//
**************************************************************************/

void t_sub1(lzo_byte *p, lzo_uint l)
{
	unsigned char b = 0;

	do {
		*p -= b;
		b += *p++;
	} while (--l > 0);
}


void t_add1(lzo_byte *p, lzo_uint l)
{
	unsigned char b = 0;

	do {
		b += *p;
		*p++ = b;
	} while (--l > 0);
}


/*************************************************************************
//
**************************************************************************/

void t_sub(lzo_byte *p, lzo_uint l, int n)
{
	unsigned char b[16];
	int i;

	assert(n > 0 && n <= (int)sizeof(b));
	if (l <= (lzo_uint)n)
		return;

	n--;
	i = n; do b[i] = 0; while (--i >= 0);

	i = n;
	do {
		*p -= b[i];
		b[i] += *p++;
		if (--i < 0)
			i = n;
	} while (--l > 0);
}


void t_add(lzo_byte *p, lzo_uint l, int n)
{
	unsigned char b[16];
	int i;

	assert(n > 0 && n <= (int)sizeof(b));
	if (l <= (lzo_uint)n)
		return;

	n--;
	i = n; do b[i] = 0; while (--i >= 0);

	i = n;
	do {
		b[i] += *p;
		*p++ = b[i];
		if (--i < 0)
			i = n;
	} while (--l > 0);
}


/*************************************************************************
//
**************************************************************************/

void t_mtf(lzo_byte *p, lzo_uint l)
{
	unsigned char b[256];
	unsigned char c;
	int i;

	for (i = 0; i < 256; i++)
		b[i] = (unsigned char) i;

	do {
		c = *p;
		for (i = 0; c != b[i]; )
			i++;
		*p++ = (unsigned char) i;
		if (i > 0)
		{
			do b[i] = b[i-1]; while (--i > 0);
			b[0] = c;
		}
	} while (--l > 0);
}


void t_unmtf(lzo_byte *p, lzo_uint l)
{
	unsigned char b[256];
	unsigned char c;
	int i;

	for (i = 0; i < 256; i++)
		b[i] = (unsigned char) i;

	do {
		i = *p;
		c = b[i];
		*p++ = c;
		if (i > 0)
		{
			do b[i] = b[i-1]; while (--i > 0);
			b[0] = c;
		}
	} while (--l > 0);
}


/*
vi:ts=4
*/

